@@ -10,6 +10,7 @@ import copy
 import datetime
 import errno
 import functools
+import inspect
 import hashlib
 import os
 import re
@@ -318,18 +319,6 @@ class Archive(object):
         if middle == ':' and prefix in HASH_ALGORITHMS:
             return prefix
 
-    def _calculate_hash(self, product, hash_type):
-        """ calculate the hash on a product in the archive """
-        product_path = self._product_path(product)
-        if product_path is None:
-            raise Error("no data available for product '%s' (%s)" % (product.core.product_name, product.core.uuid))
-
-        plugin = self.product_type_plugin(product.core.product_type)
-
-        product_hash = functools.partial(util.product_hash, hash_type=hash_type)
-
-        return self._storage.run_for_product(product, product_hash, plugin.use_enclosing_directory)
-
     def _establish_invariants(self):
         repeat = True
         cycle = 0
@@ -396,10 +385,11 @@ class Archive(object):
         plugin = self.product_type_plugin(product.core.product_type)
         self._run_hooks('post_remove_hook', plugin, product, reverse=True)
 
-    def _relocate(self, product, properties=None):
+    def _relocate(self, product, properties=None, paths=None):
         """Relocate a product to the archive_path reported by the product type plugin.
-        Returns the new archive_path if the product was moved."""
-        result = None
+        Returns the new archive_path if the product was moved and optionally updated
+        (local) product paths.
+        """
         product_archive_path = product.core.archive_path
         if properties:
             product = copy.deepcopy(product)
@@ -408,8 +398,10 @@ class Archive(object):
         plugin_archive_path = plugin.archive_path(product)
 
         if product_archive_path != plugin_archive_path:
-            self._storage.move(product, plugin_archive_path)
-            return plugin_archive_path
+            paths = self._storage.move(product, plugin_archive_path, paths)
+            return plugin_archive_path, paths
+        else:
+            return None, paths
 
     def _remove(self, product):
         # If the product has no data in storage associated with it, return.
@@ -815,7 +807,7 @@ class Archive(object):
                         raise Error("ingested product has incorrect hash")
 
                 properties.core.archive_date = self._database.server_time_utc()
-        except:
+        except Exception:
             # Try to remove the entry for this product from the product catalogue.
             self._database.delete_product_properties(properties.core.uuid)
             raise
@@ -835,7 +827,7 @@ class Archive(object):
         if not ingest_product:
             self._run_hooks('post_create_hook', plugin, properties)
         else:
-            self._run_hooks('post_ingest_hook', plugin, properties)
+            self._run_hooks('post_ingest_hook', plugin, properties, paths=paths)
 
         return properties
 
@@ -846,7 +838,7 @@ class Archive(object):
                 return True
         return False
 
-    def _run_hooks(self, hook_name, plugin, properties, reverse=False):
+    def _run_hooks(self, hook_name, plugin, properties, reverse=False, paths=None):
         plugins = [plugin] + list(self._hook_extensions.values())
         if reverse:
             plugins = reversed(plugins)
@@ -854,7 +846,15 @@ class Archive(object):
         for plugin in plugins:
             hook_method = getattr(plugin, hook_name, None)
             if hook_method is not None:
-                hook_method(self, properties)
+                try:
+                    getargspec = inspect.getfullargspec
+                except AttributeError:
+                    getargspec = inspect.getargspec
+
+                if len(getargspec(hook_method).args) == 4:
+                    hook_method(self, properties, paths)
+                else:
+                    hook_method(self, properties)
 
     def link(self, uuid_, source_uuids):
         """Link a product to one or more source products."""
@@ -917,7 +917,7 @@ class Archive(object):
 
         return os.path.join(self._storage.global_prefix, self._product_path(product))
 
-    def pull(self, where="", parameters={}, verify_hash=False):
+    def pull(self, where="", parameters={}, verify_hash=False, verify_hash_download=False):
         """Pull one or more remote products into the archive.
         Return the number of products pulled.
 
@@ -929,6 +929,8 @@ class Archive(object):
         parameters    --  Parameters referenced in the search expression (if any).
         verify_hash   --  If set to True then, after the pull, the product in the archive will be matched against
                           the hash from the metadata (only if the metadata contained a hash).
+        verify_hash_download  --  If set to True then, before the product is stored in the archive, the pulled
+                          product will be matched against the metadata hash (if it exists).
 
         """
         queue = self.search(where=where, parameters=parameters, namespaces=self.namespaces())
@@ -950,30 +952,33 @@ class Archive(object):
             metadata = {'active': False, 'archive_path': product.core.archive_path}
             self.update_properties(Struct({'core': metadata}), product.core.uuid)
 
+            def _pull(paths):
+                # reactivate and update size
+                product_path = self._product_path(product)
+                size = self._storage.size(product_path)
+                metadata = {'active': True, 'archive_date': self._database.server_time_utc(), 'size': size}
+                self.update_properties(Struct({'core': metadata}), product.core.uuid)
+
+                # verify product hash.
+                if verify_hash and 'hash' in product.core:
+                    if self.verify_hash("uuid == @uuid", {"uuid": product.core.uuid}):
+                        raise Error("pulled product '%s' (%s) has incorrect hash" %
+                                    (product.core.product_name, product.core.uuid))
+
+                # Run the post pull hook (if defined by the product type plug-in or hook extensions).
+                self._run_hooks('post_pull_hook', plugin, product, paths=paths)
+
             # pull product
             try:
-                remote.pull(self, product, use_enclosing_directory)
-            except:
+                retrieve_files = remote.retrieve_function(self, product, verify_hash_download)
+                self._storage.put(None, product, use_enclosing_directory, use_symlinks=False,
+                                  retrieve_files=retrieve_files, run_for_product=_pull)
+            except Exception:
                 # reset active/archive_path values
                 metadata = {'active': True, 'archive_path': None}
                 self.update_properties(Struct({'core': metadata}), product.core.uuid)
                 raise
 
-            # reactivate and update size
-            product_path = self._product_path(product)
-            size = self._storage.size(product_path)
-            metadata = {'active': True, 'archive_date': self._database.server_time_utc(), 'size': size}
-            self.update_properties(Struct({'core': metadata}), product.core.uuid)
-
-            # verify product hash.
-            if verify_hash and 'hash' in product.core:
-                if self.verify_hash("uuid == @uuid", {"uuid": product.core.uuid}):
-                    raise Error("pulled product '%s' (%s) has incorrect hash" %
-                                (product.core.product_name, product.core.uuid))
-
-            # Run the post pull hook (if defined by the product type plug-in or hook extensions).
-            self._run_hooks('post_pull_hook', plugin, product)
-
         return len(queue)
 
     def rebuild_properties(self, uuid, disable_hooks=False, use_current_path=False):
@@ -1004,61 +1009,64 @@ class Archive(object):
         plugin = self.product_type_plugin(product.core.product_type)
         use_enclosing_directory = plugin.use_enclosing_directory
 
-        metadata = self._storage.run_for_product(product, plugin.analyze, use_enclosing_directory)
+        def _rebuild_properties(paths):
+            metadata = plugin.analyze(paths)
 
-        if isinstance(metadata, (tuple, list)):
-            properties, tags = metadata
-        else:
-            properties, tags = metadata, []
-
-        # Remove properties that should not be changed.
-        assert "core" in properties
-        for name in restricted_properties:
-            try:
-                delattr(properties.core, name)
-            except AttributeError:
-                pass
-
-        # update size
-        properties.core.size = self._storage.size(product_path)
-
-        # Make sure product is stored in the correct location
-        if not use_current_path:
-            new_archive_path = self._relocate(product, properties)
-            if new_archive_path:
-                properties.core.archive_path = new_archive_path
-
-        # if product type has disabled hashing, remove existing hash values
-        stored_hash = getattr(product.core, 'hash', None)
-        plugin_hash_type = self._plugin_hash_type(plugin)
-
-        if plugin_hash_type is None:
-            if stored_hash is not None:
-                properties.core.hash = None
-
-        # if product type has different hash algorithm, update hash values
-        else:
-            if stored_hash is None:
-                properties.core.hash = self._calculate_hash(product, plugin_hash_type)
+            if isinstance(metadata, (tuple, list)):
+                properties, tags = metadata
             else:
-                hash_type = self._extract_hash_type(stored_hash)
-                if hash_type is None and plugin_hash_type == 'sha1':
-                    properties.core.hash = plugin_hash_type + ':' + stored_hash
-                elif hash_type != plugin_hash_type:
-                    properties.core.hash = self._calculate_hash(product, plugin_hash_type)
+                properties, tags = metadata, []
 
-        # Update product properties.
-        self.update_properties(properties, uuid=product.core.uuid, create_namespaces=True)
+            # Remove properties that should not be changed.
+            assert "core" in properties
+            for name in restricted_properties:
+                try:
+                    delattr(properties.core, name)
+                except AttributeError:
+                    pass
 
-        # Update tags.
-        self.tag(product.core.uuid, tags)
+            # update size
+            properties.core.size = self._storage.size(product_path)
 
-        # Run the post ingest hook (if defined by the product type plug-in or hook extensions).
-        if not disable_hooks and self._hook_exists('post_ingest_hook', plugin):
-            product.update(properties)
-            if 'hash' not in product.core:
-                product.core.hash = None
-            self._run_hooks('post_ingest_hook', plugin, product)
+            # Make sure product is stored in the correct location
+            if not use_current_path:
+                new_archive_path, paths = self._relocate(product, properties, paths)
+                if new_archive_path is not None:
+                    properties.core.archive_path = new_archive_path
+
+            # if product type has disabled hashing, remove existing hash values
+            stored_hash = getattr(product.core, 'hash', None)
+            plugin_hash_type = self._plugin_hash_type(plugin)
+
+            if plugin_hash_type is None:
+                if stored_hash is not None:
+                    properties.core.hash = None
+
+            # if product type has different hash algorithm, update hash values
+            else:
+                if stored_hash is None:
+                    properties.core.hash = util.product_hash(paths, hash_type=plugin_hash_type)
+                else:
+                    hash_type = self._extract_hash_type(stored_hash)
+                    if hash_type is None and plugin_hash_type == 'sha1':
+                        properties.core.hash = plugin_hash_type + ':' + stored_hash
+                    elif hash_type != plugin_hash_type:
+                        properties.core.hash = util.product_hash(paths, hash_type=plugin_hash_type)
+
+            # Update product properties.
+            self.update_properties(properties, uuid=product.core.uuid, create_namespaces=True)
+
+            # Update tags.
+            self.tag(product.core.uuid, tags)
+
+            # Run the post ingest hook (if defined by the product type plug-in or hook extensions).
+            if not disable_hooks and self._hook_exists('post_ingest_hook', plugin):
+                product.update(properties)
+                if 'hash' not in product.core:
+                    product.core.hash = None
+                self._run_hooks('post_ingest_hook', plugin, product, paths=paths)
+
+        self._storage.run_for_product(product, _rebuild_properties, use_enclosing_directory)
 
     def rebuild_pull_properties(self, uuid, verify_hash=False, disable_hooks=False, use_current_path=False):
         """Refresh products by re-running the pull, but using the existing products stored in the archive.
@@ -1078,11 +1086,12 @@ class Archive(object):
             raise Error("cannot pull products that have no remote_url")
 
         plugin = self.product_type_plugin(product.core.product_type)
+        use_enclosing_directory = plugin.use_enclosing_directory
 
         # make sure product is stored in the correct location
         if not use_current_path:
-            new_archive_path = self._relocate(product)
-            if new_archive_path:
+            new_archive_path, _ = self._relocate(product)
+            if new_archive_path is not None:
                 metadata = {'archive_path': new_archive_path}
                 self.update_properties(Struct({'core': metadata}), product.core.uuid)
                 product.core.archive_path = new_archive_path
@@ -1091,15 +1100,18 @@ class Archive(object):
         product_path = self._product_path(product)
         product.core.size = self._storage.size(product_path)
 
-        # verify product hash.
-        if verify_hash and 'hash' in product.core:
-            if self.verify_hash("uuid == @uuid", {"uuid": product.core.uuid}):
-                raise Error("pulled product '%s' (%s) has incorrect hash" %
-                            (product.core.product_name, product.core.uuid))
+        def _rebuild_pull_properties(paths):
+            # verify product hash.
+            if verify_hash and 'hash' in product.core:
+                if not self._verify_hash(product, paths):
+                    raise Error("pulled product '%s' (%s) has incorrect hash" %
+                                (product.core.product_name, product.core.uuid))
 
-        # Run the post pull hook (if defined by the product type plug-in or hook extensions).
-        if not disable_hooks:
-            self._run_hooks('post_pull_hook', plugin, product)
+            # Run the post pull hook (if defined by the product type plug-in or hook extensions).
+            if not disable_hooks:
+                self._run_hooks('post_pull_hook', plugin, product, paths=paths)
+
+        self._storage.run_for_product(product, _rebuild_pull_properties, use_enclosing_directory)
 
     def remove(self, where="", parameters={}, force=False):
         """Remove one or more products from the archive, both from storage as well as from the product catalogue.
@@ -1413,6 +1425,35 @@ class Archive(object):
         self._update_metadata_date(properties)
         self._database.update_product_properties(properties, uuid=uuid, new_namespaces=new_namespaces)
 
+    def _verify_hash(self, product, paths=None):
+        if 'archive_path' in product.core:
+            if 'hash' not in product.core:
+                raise Error("no hash available for product '%s' (%s)" %
+                            (product.core.product_name, product.core.uuid))
+
+            product_path = self._product_path(product)
+            if product_path is None:
+                raise Error("no data available for product '%s' (%s)" % (product.core.product_name, product.core.uuid))
+
+            stored_hash = product.core.hash
+            hash_type = self._extract_hash_type(stored_hash)
+            plugin = self.product_type_plugin(product.core.product_type)
+
+            if hash_type is None:
+                hash_type = 'sha1'  # default before use_hash deprecation
+                stored_hash = 'sha1:' + stored_hash
+
+            if paths is None:
+                product_hash = functools.partial(util.product_hash, hash_type=hash_type)
+                current_hash = self._storage.run_for_product(product, product_hash, plugin.use_enclosing_directory)
+            else:
+                current_hash = util.product_hash(paths, hash_type=hash_type)
+
+            if current_hash != stored_hash:
+                return False
+
+        return True
+
     def verify_hash(self, where="", parameters={}):
         """Verify the hash for one or more products in the archive.
         Returns a list of UUIDs of products for which the verification failed.
@@ -1431,22 +1472,8 @@ class Archive(object):
                                property_names=['uuid', 'active', 'product_name', 'archive_path', 'physical_name',
                                                'hash', 'product_type'])
         for product in products:
-            if 'archive_path' in product.core:
-                if 'hash' not in product.core:
-                    raise Error("no hash available for product '%s' (%s)" %
-                                (product.core.product_name, product.core.uuid))
-
-                stored_hash = product.core.hash
-                hash_type = self._extract_hash_type(stored_hash)
-
-                if hash_type is None:
-                    hash_type = 'sha1'  # default before use_hash deprecation
-                    stored_hash = 'sha1:' + stored_hash
-
-                current_hash = self._calculate_hash(product, hash_type)
-
-                if current_hash != stored_hash:
-                    failed_products.append(product.core.uuid)
+            if not self._verify_hash(product):
+                failed_products.append(product.core.uuid)
 
         return failed_products
 
