@@ -8,6 +8,7 @@ from muninn._compat import dictkeys, dictvalues, is_python2_unicode
 
 import re
 import functools
+import inspect
 
 try:
     import psycopg2
@@ -45,7 +46,7 @@ def _get_db_type_id(connection, typename):
             type_id = cursor.description[0][1]
         finally:
             cursor.close()
-    except:
+    except Exception:
         connection.rollback()
         raise
     else:
@@ -54,7 +55,7 @@ def _get_db_type_id(connection, typename):
     return type_id
 
 
-def geometry_recv(data, offset, length): # TODO binary send/recv needed for pg8000 <= 1.15
+def geometry_recv(data, offset, length):  # TODO binary send/recv needed for pg8000 <= 1.15
     return ewkb.decode_ewkb(data[offset:offset+length])
 
 
@@ -90,7 +91,15 @@ def _connect_pg8000(connection_string):
         geometry.MultiLineString,
     ):
         if hasattr(_connection, 'register_out_adapter'):
-            _connection.register_out_adapter(type_, geography_oid, geometry_send_hex)
+            try:
+                getargspec = inspect.getfullargspec
+            except AttributeError:
+                getargspec = inspect.getargspec
+            # pg8000 removed the oid argument in v1.22
+            if len(getargspec(_connection.register_out_adapter).args) == 3:
+                _connection.register_out_adapter(type_, geometry_send_hex)
+            else:
+                _connection.register_out_adapter(type_, geography_oid, geometry_send_hex)
         else:
             _connection.py_types[type_] = (geography_oid, pg8000.core.FC_BINARY, geometry_send)
 
@@ -310,7 +319,6 @@ class PostgresqlBackend(object):
                     result.append("CREATE INDEX idx_%s_%s ON %s (%s)" %
                                   (self._core_table_name, name, self._core_table_name, name))
 
-
         # Create the tables for all non-core namespaces.
         for namespace in self._namespace_schemas:
             if namespace == "core":
