@@ -194,6 +194,10 @@ def archive(database, storage, use_enclosing_directory, archive_path):
         del sys.modules['hook_extension']
     os.system('rm *.pyc -f')
 
+    # create empty dirs (can't commit in git)
+    os.makedirs('data/multi/emptydir', exist_ok=True)
+    os.makedirs('data/multi/dir/emptydir', exist_ok=True)
+
     # create clean archive
     with muninn.open('my_arch') as archive:
         archive.register_namespace('mynamespace', MyNamespace)
@@ -225,8 +229,7 @@ class TestArchive:
                 'one/two'
             )
 
-            if not os.path.isdir(dirpath):
-                os.makedirs(dirpath)
+            os.makedirs(dirpath, exist_ok=True)
             shutil.copy(path, dirpath)
             path = os.path.join(dirpath, name)
 
@@ -285,10 +288,12 @@ class TestArchive:
                 'three/multi'
             )
 
-            if not os.path.isdir(dirpath):
-                os.makedirs(dirpath)
+            os.makedirs(dirpath, exist_ok=True)
             for path in paths:
-                shutil.copy(path, dirpath)
+                if os.path.isdir(path):
+                    shutil.copytree(path, os.path.join(dirpath, os.path.basename(path)))
+                else:
+                    shutil.copy(path, dirpath)
 
             paths = [os.path.join(dirpath, os.path.basename(path)) for path in paths]
 
@@ -309,7 +314,7 @@ class TestArchive:
                     'multi',
                     os.path.basename(path)
                 )
-                assert os.path.isfile(os.path.realpath(source_path))
+                assert os.path.exists(os.path.realpath(source_path))
 
                 if intra:
                     # TODO remove this, as os.path.realpath already resolves
@@ -371,7 +376,7 @@ class TestArchive:
 
         archive.update_properties(muninn.Struct({'core': metadata}), props.core.uuid)
 
-        archive.pull("", verify_hash=True)
+        archive.pull("", verify_hash=True, verify_hash_download=True)
 
         path = os.path.join(archive._params['archive_path'], 'README')
         if archive._params['use_enclosing_directory']:
@@ -418,6 +423,9 @@ class TestArchive:
 
             # force reingest _from archive_ (should not remove product!)
             product_path = archive.product_path(properties)
+            if archive._params['use_enclosing_directory']:
+                product_path = os.path.join(product_path, 'pi.txt')
+
             properties = archive.ingest(product_path, force=True)
             assert os.path.exists(product_path)
 
@@ -434,7 +442,6 @@ class TestArchive:
     def test_ingest_multi_file(self, archive):
         # copy
         self._ingest_multi_file(archive)
-
         archive.remove()
 
         # symlink
@@ -785,7 +792,12 @@ class TestArchive:
         # aggregate size.sum
         data, headers = archive.summary(aggregates=['core.size.sum']) # TODO doesn't work without core prefix
         assert headers == ['count', 'core.size.sum']
-        assert data == [(2, 2030)] or data == [[2, 2030]]
+        assert data == [(2, 2030)] or data == [[2, 2030]] # TODO pg8000
+
+        # non-core namespace
+        data, headers = archive.summary(aggregates=['mynamespace2.counter.avg'])
+        assert headers == ['count', 'mynamespace2.counter.avg']
+        assert data == [(2, 28,)] or data == [[2, 28]]  # TODO pg8000
 
         # group by archive date plus subscript
         for subscript in [
